<?php
	/**
	 *	The WebDAV class (extends {@link HTTP_Server})
	 *	
	 *	:-)
	 *	
	 *	@package	webdav
	 *	@version	0.5
	 *	@author		Stepan Kouba <stepankouba@formiginal.com>
	 */
	
	
	// import http_server class
	require_once( "http_server.php" );
	
	/**
	 *	DAV_0
	 *	Constant representing value 0 for Depth header (use only with <strong>setDepht</strong> method)
	 */
	define( "DAV_0", 0 );
	
	/**
	 *	DAV_1
	 *	Constant representing value 1 for Depth header (use only with <strong>setDepht</strong> method)
	 */
	define( "DAV_1", 1 );
	
	/**
	 *	DAV_INFINITY
	 *	Constant representing value Infinity for Depth header (use only with <strong>setDepht</strong> method)
	 */
	define( "DAV_INFINITY", "Infinity" );
	
	class WebDAV extends HTTP_Server
	{
		/**
		 *	description of error, which have occured during operation
		 *	@var	string
		 */
		var $error;
		
		/**
		 *	value of depth header. This variable must be empty after calling every method
		 *	@var	mixed
		 */
		var $depth = false;
		
		/**
		 *	call parent constructor
		 *	
		 *	@param	string	$host	hosting server
		 *	@param	int		$port	http server's port
		 */
		function WebDAV( $host, $port = 80 )
		{
			parent::HTTP_Server( $host, $port );
		}// function
		
		/**
		 *	This method tries to copy source to destination. Third optional parametr is used to set whether destination (if already
		 *	exists) should be overwrite or not.
		 *	Status codes which server may return in case of success are: 204 (overwriten old file) or 201 (created new file).
		 *	
		 *	Caution: <strong>Copying collections does not support sending XML document, which specifies behaviot of properties.</strong>
		 *	
		 *	@param	string	$source			absolute path to the source object
		 *	@param	string	$destination	complete URI of destination (for example: http://localhost/utils/index.php)
		 *	@param	bool	$overwrite		whether to use <strong>Overwrite</strong> header or not
		 *	
		 *	@return	bool
		 */
		function copy( $source, $destination, $overwrite = true )
		{
			// set overwrite header only if do not want to use it. Otherwise it will be used.
			if ( !$overwrite )
				$this->setHeader( "Overwrite", "F" );
			
			// set destination header
			$this->setHeader( "Destination", $destination );
			
			// set depth header if needed so
			if ( !is_bool( $this->depth ) )
				$this->setHeader( "Depth", $this->depth );
			
			// connect
			$this->connect();
			
			// send request
			$this->request( "COPY", $source );
			
			// wait for response
			$this->response();
			
			// close connection
			$this->close();
			
			// empty depth variable
			$this->depth = false;
			
			// check status code and in case of an error return false and save the error
			$status = $this->getStatus();
			if ( $status["code"] == "204" || $status["code"] == "201" )
				return true;
			else
			{
				$this->error = $status["phrase"];
				return false;
			}// else
		}// function
		
		/**
		 *	Simply deletes source and return boolean according to how the operation resulted.
		 *	In case of an error is (as usual in this class) saved in {@link $this->error variable}. If status code is 207 (Multi-Status), than
		 *	is saved in this variable whole XML document describing error/s.
		 *	
		 *	@param	string	$path	absolute path to the source
		 *	@return	bool
		 */
		function delete( $path )
		{
			// connect server
			$this->connect();
			
			// send request
			$this->request( "DELETE", $path );
			
			// wait for response
			$this->response();
			
			// close connection
			$this->close();
			
			// check response status code and in case of 207 save whole XML document
			$status = $this->getStatus();
			if ( $status["code"] == 204 )
				return true;
			elseif ( $status["code"] == 207 )
			{
				$this->error = $this->getServerBody();
				return false;
			}// elseif
			else
			{
				$this->error = $status["phrase"];
				return false;
			}// else
		}// function
		
		/**
		 *	creates new collection which is specified with absolute path.
		 *	
		 *	Caution: <strong>Every ancestor must already exist!</strong>
		 *	
		 *	@param	string	$path	absolute path of new collection
		 *	@return	bool
		 */
		function mkcol( $path )
		{
			// connect http server
			$this->connect();
			
			// send request
			$this->request( "MKCOL", $path );
			
			// wait for response
			$this->response();
			
			// close connection
			$this->close();
			
			// status code and phrase
			$status = $this->getStatus();
			if ( $status["code"] == 201 )
				return true;
			else
			{
				$this->error = $status["phrase"];
				return false;
			}// else
		}// function
		
		/**
		 *	use this method to move source to destination. Be carefull when setting {@link $destination}. MOVE method can return
		 *	201 or 204 status codes in case of success. Other status codes are returned in case of error. When status code 207 was
		 *	returned then response contain a XML document describing response (but this may occur when collections are moved).
		 *	
		 *	Caution: <strong>At this time it works only with files and collections, which are not locked!</strong>
		 *	Caution: <strong>Moving collections does not support sending XML document, which specifies behaviot of properties.</strong>
		 *	
		 *	@param	string	$source		absolute path to the source (for example: /utils/index.php)
		 *	@param	string	$destination	aboslute destination path with file name and server name (for example: http://localhost/utils/new/index.php)
		 *	@param	bool	$overwrite	whether to use <strong>Overwrite</strong> header or not
		 *	@return	bool
		 */
		function move( $source, $destination, $overwrite = true )
		{
			// overwrite existing destination
			if ( !$overwrite )
				$this->setHeader( "Overwrite", "F" );
			
			// set destionation header
			$this->setHeader( "Destination", $destination );
			
			// connect server
			$this->connect();
			
			// send request
			$this->request( "MOVE", $source );
			
			// wsit for response
			$this->response();
			
			// close connection
			$this->close();
			
			// get status
			$status = $this->getStatus();
			if ( $status["code"] == 201 || $status["code"] == 204 )
				return true;
			elseif ( $status["code"] == 207 )
			{
				$this->error = $this->getServerBody();
				return false;
			}// elseif
			else
			{
				$this->error = $status["phrase"];
				return false;
			}// else
		}// function
		
		/**
		 *	send PROPFIND request and create XML document, which will be send with this HTTP method
		 *	
		 *	@param	string	$path		absolute path to the source
		 *	@param	bool	$getNamesOnly	create HTTP request, that will return only names of properties
		 *	@param	array	$props		array of properties, their values will server return
		 *	@param	array	$ns			namespaces which are used in list of properties user want to get
		 *	@return	string	content of server's response (XML document)
		 */
		function propfind( $source, $getNamesOnly = false, $props = "", $ns = "" )
		{
			// create beginning of XML document
			$xml = 	"<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n".
					"<D:propfind xmlns:D=\"DAV:\">\n";
			
			// create XML document according to what user want to
			if ( !$getNamesOnly && !is_array( $props ) )
				$xml .= "	<D:allprop/>\n";
			elseif ( $getNamesOnly && !is_array( $props ) )
				$xml .= "	<D:propname/>\n";
			else
			{
				$xml .= "	<D:prop";
				
				// user use some namespaces
				if ( is_array( $ns ) )
				{
					foreach ( $ns as $k => $n )
					{
						$xml .= " xmlns:".$k."=\"".$n."\"";
					}// foreach
				}// if
				
				$xml .= ">\n";
				
				// wanted properties
				foreach ( $props as $prop )
				{
					$xml .= "		<".$prop."/>\n";
				}// foreach
				
				$xml .= "	<D:prop/>";
			}// else
			
			$xml .= "</D:propfind>";
			
			// set depth header if needed so
			if ( !is_bool( $this->depth ) )
				$this->setHeader( "Depth", $this->depth );
				
			// set client body and type
			$this->setClientBody( $xml, "text/xml" );
			
			// connect to HTTP server
			$this->connect();
			
			// send request
			$this->request( "PROPFIND", $source );
			// wait for response
			
			$this->response();
			
			// close connection
			$this->close();
			
			// empty depth variable
			$this->depth = false;
			
			// return XML document
			return $this->getServerBody();
		}// function
		
		/**
		 * 	use this method to set or/and remove properties for file/collection. First parameter must be set. It is absolute path to
		 *	a file/collection. In associative array <strong>$set</strong> and array <strong>$remove</strong> should be set properties
		 *	and (in case you want to set some properties) their values. Last parameter is used to set obligatory namespaces if needed.
		 *	
		 *	@param	string	$path	absolute path to the source
		 *	@param	array	$set	associative array of properties and their values ("weather" => "good"), which have to be set
		 *	@param	array	$remove	array of properties to remove
		 *	@param	array	$ns		associative	 array of namespaces ("R" => "http://www.webdav.com/dtd/one")
		 *	
		 *	@return	string	XML document
		 */
		function proppatch( $source, $set = "", $remove = "", $ns = "" )
		{
			// create beginning of XML document
			$xml = 	"<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n".
					"<D:propertyupdate xmlns:D=\"DAV:\"";
			
			// adding namespaces
			if ( is_array( $ns ) )
			{
				foreach ( $ns as $k => $n )
				{
					$xml .= " xmlns:".$k."=\"".$n."\"";
				}// foreach
			}// if
			
			$xml .=	">\n";
			
			// set properties
			if ( is_array( $set ) )
			{
				$xml .= "	<D:set>\n";
				$xml .= "		<D:prop>\n";
				
				foreach ( $set as $k => $s )
				{
					$xml .=	"			<".$k.">".$s."</".$k.">";
				}// foreach
				
				$xml .= "		</D:prop>\n";
				$xml .= "	</D:set>\n";
			}// if
			
			// remove properties
			if ( is_array( $remove ) )
			{
				$xml .= "	<D:remove>\n";
				$xml .= "		<D:prop>\n";
				
				foreach ( $remove as $r )
				{
					$xml .=	"			<".$r."/>";
				}// foreach
				
				$xml .= "		</D:prop>\n";
				$xml .= "	</D:remove>\n";
			}// if
			
			$xml .= "</D:propertyupdate>\n";
			
			// set depth header
			if ( !is_bool( $this->depth ) )
				$this->setHeader( "Depth", $this->depth );
			
			// set client body and type
			$this->setClientBody( $xml, "text/xml" );
			
			// connect to HTTP server
			$this->connect();
			
			// send request
			$this->request( "PROPPATCH", $source );
			
			// wait for response
			$this->response();
			
			// close connection
			$this->close();
			
			// empty depth variable
			$this->depth = false;
			
			// return XML document
			return $this->getServerBody();
		}// function
		
		/**
		 *	lock method is used to lock files/collections. If we try to lock a file, then server may return these status codes: 200,
		 *	412 or 423. Only the first stands for success. If we try to lock collections, then 207 status code is returned. In case of
		 *	207 and 200 method will return content of server's response (XML document), otherwise it will return false and save error.
		 *	
		 *	Caution: <strong>At this time this method does not support authorization!</strong>
		 *	Caution: <strong>Now it does not support refreshing locks (sending <strong>If</strong> header) too.</strong>
		 *	
		 *	@param	string	$path	absolute path to the source
		 *	@param	mixed	$time	content of timeout header, which may content following values seperated with comma: <strong>Infinty</strong> or <strong>number of seconds-</strong>
		 *	@param	string	$scope	lock scope (<strong>exclusive</strong> or <strong>shared</strong>)
		 *	@param	string	$owner	URI of owner homepage
		 *	
		 *	@deprecated	{@link $owner} will no longer be supported. It will be replaced with object containing information about owner
		 *	
		 *	@return mixed	XML document or boolean
		 */
		function lock( $source, $time, $scope = "exclusive", $owner = "" )
		{
			// create beginning of XML document
			$xml = 	"<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n".
					"<D:lockinfo xmlns:D=\"DAV:\">\n".
					"	<D:lockscope><D:".$scope."/></D:lockscope>\n".
					"	<D:locktype><D:write/></D:locktype>\n";
			
			// add owner href if set
			if ( $owner )
				$xml .=	"	<D:owner><D:href>".$owner."</D:href></D:owner>\n";
			
			// end creation of XML document
			$xml .= "</D:lockinfo>";
			
			// set client body
			$this->setClientBody( $xml, "text/xml" );
			
			// send header timeout
			$this->setHeader( "Timeout", $time );
			
			// set Depth header if user want to
			if ( !is_bool( $this->depth ) )
				$this->setHeader( "Depth", $this->depth );
			
			// connect server
			$this->connect();
			
			// send request
			$this->request( "LOCK", $source );
			
			// wait for response
			$this->response();
			
			// close connection
			$this->close();
			
			// empty depth variable
			$this->depth = false;
			
			// get status code and return XML document if everything OK, elsewhere return false and save error code
			$status = $this->getStatus();
			if ( $status["code"] == 200 || $status["code"] == 207 )
				return $this->getServerBody();
			else
			{
				$this->error = $status["phrase"];
				return false;
			}// else
		}// function
		
		/**
		 *	unlock already locked source (file or collection). Proper lock token must be send to unlock the source.
		 *
		 *	Caution: <strong>As {@link WebDAV::lock()} this method does not support authorization!</strong>
		 *	
		 *	@param	string	$path		absolute path to the source
		 *	@param	string	$lockToken	lock token by which is locked source identified (get it from element <strong>locktoken</strong>)
		 *	@return bool
		 */
		function unlock( $source, $lockToken )
		{
			// send lock token
			$this->setHeader( "Lock-Token", "<".$lockToken.">" );
			
			// connect server
			$this->connect();
			
			// send request
			$this->request( "UNLOCK", $source );
			
			// wait for response
			$this->response();
			
			// close connection
			$this->close();
			
			// get status
			$status = $this->getStatus();
			if ( $status["code"] == 204 )
				return true;
			else
			{
				$this->error = $status["phrase"];
				return false;
			}// else
		}// function
		
		/**
		 *	some methods may be used together with collections. <strong>Depth</strong> header influences behaviour of methods
		 *	like propfind, copy, move, ...
		 *	Possible values of <strong>Depth</strong> header: 0, 1 or Infinity
		 *	
		 *	@param	mixed	$depth	value of <strong>Depth</strong> header which may be used with some methods (copy, move, propfind)
		 */
		function setDepth( $depth )
		{
			$this->depth = $depth;
		}// function
	}// class
?>